'**************************************************************************
' Vmux55_V32   MAX4355 Vkreuzschienen Steuerung                22.12.2015 *
'**************************************************************************

'**************************************************************************
'*Allgemeine Definitionen
'**************************************************************************
$regfile = "M8def.dat"
$crystal = 8000000                                ' interner RC-Oszillator
$baud = 38400

Debug Off                                         ' nur zum debuggen
'Osccal = &HBF


$framesize = 32                                   '
$swstack = 32                                     '
$hwstack = 32                                     '

'**************************************************************************
'* LCD-Einstellungen
'**************************************************************************
Config Lcdpin = Pin , Rs = Portb.0 , E = Portb.1 , Db4 = Portb.2 , Db5 = Portb.3 , Db6 = Portb.4 , Db7 = Portb.5
Config Lcdbus = 4
Config Lcd = 16x2

'**************************************************************************
'* TWI Slave config
'**************************************************************************
Config Twislave = &H62 , Btr = 16 , Bitrate = 100000       ' Adresse = 98 / bis 16 bytes abrufbar / 100kHz
$lib "i2c_twi.lbx"                                ' ist Teil der "i2cslave.lib"
Config Scl = Portc.5                              '
Config Sda = Portc.4
Portc.5 = 0
Portc.4 = 0

'**************************************************************************
'* Port config: C.4 und C.5 fr i2c slave freigehalten / Layout vom Herwig
'**************************************************************************
Config Portc = &B0000_1111                        ' 0 - 3 output
Din Alias Portc.0
Maxmode Alias Portc.1
Ce Alias Portc.2
Sclk Alias Portc.3

Config Portd = &B0100_0000                        ' alle input auer D.6
Auf Alias Pind.0                                  ' fr Tasten Auf/Ab
Ab Alias Pind.1                                   ' D.0 und D.1 nur fr STK200, knnten entfallen
                 ' Pind.2 (INT0)  fr Encoder "A"
Richtung Alias Pind.3                             ' Schalter am Encoder
Updown Alias Pind.4                               ' Encoder Pin B
Update Alias Portd.6                              ' output !
Aout Alias Pind.7                                 ' Rckmeldung, da Adresse OK = 0

Portd = &B1011_1111                               ' pullups einschalten

Config Int0 = Change                              ' an Pind.2 auf Flanke warten

'*************************************************************************
'* Variable
'*************************************************************************
Dim I As Byte , N As Byte                         ' Zhlvariable
Dim V As Byte                                     ' Index fr Videomatrix/Array
Dim Chout(16) As Byte                             ' Ausgangskanalnummer
Dim Chout_asc As Byte                             ' als Ascii Zahl
Dim Chout_disp As String * 2                      ' zur LCD Darstellung
Dim Inout As Bit                                  ' Flag fr Input(1) / Output(0)
Dim Z As Byte                                     ' Displayzeile

Dim Maske As Word                                 ' fr Mode 0 Ausgabe
Dim 16bitdaten As Word                            ' dto.

Dim 6bitdaten As Byte                             ' Initial setup (mode 1)
Dim Masterbyte As Byte                            ' fr TWI slave
Dim Chin As Byte
Dim Schalter As Byte                              ' 3. byte per I2C ist Schalter
Dim Byte_count As Byte                            ' count bytes received as slave

Dim Neue_daten As Byte                            ' flag fr 3 bytes vom Master

Dim I_old As Byte                                 ' als Merker fr die Position

Dim Plus As Bit                                   ' als Interrupt flags
Dim Minus As Bit                                  '


'*************************************************************************
'* Defaultwerte zur Initialisierung der Eingabekanle
Dta:
$eeprom
Data 0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 1 , 1 , 1 , 1 , 1 , 1 , 1
$data
'*************************************************************************


'*************************************************************************
'* Initialisierung:
'*************************************************************************
'$sim                                              ' Debug
'Osccal = &HBF                                     ' fr "exakte" 8MHz wg. UART fr Debug
'Ucsrb = 0                                         ' UART Pins abschalten NICHT bei Bootloader-Betrieb
                                                   ' !!!!!!!!!!!!!!!!!!!!!!!!!!!
Cls
Waitms 25
Cursor Off
Lcd "VmuxV32"
Debug "Vmux_V32 Bootloader-Version"               ' I'm alive !
Wait 2

Byte_count = 0                                    ' zhlt empfangene bytes
Inout = 0
Z = 2                                             ' Cursor Zeile 2
I = 1                                             ' ... und Position 1
Cls
Lowerline
Lcd "1234567890123456"                            ' sind die Ausgabekanle 1-16
Home

Gosub Standard_laden                              '  jetzt aus EEPROM
Gosub 96bitausgabe1                               ' und jetzt zum chip
Gosub Erste_zeile


Enable Int0                                       ' Encoder Auswertung
Enable Interrupts                                 ' fr Encoder und TWI

'*************************************************************************
'* Hauptprogramm:
'*************************************************************************
On Int0 Isr0                                      ' bei Flankenwechsel zur Auswertung der Richtung

I = 1                                             ' Position in der LCD-Zeile
Cursor On Blink

Locate Z , I                                      ' Chout blinkt

Do                                                ' Falls Tasten statt Encoder
   Debounce Richtung , 0 , Einaus , Sub           ' Pind.3 / toggle Eingabe/Ausgabe
   If Plus = 1 Then Gosub Plusplus                ' Auswertung der Interrupt flags
   If Minus = 1 Then Gosub Minusminus
   If Neue_daten = 1 Then Gosub Daten_auswertung  ' Auswertung der 3 neuen bytes

   'Gosub Twi_gotdata                              ' DEBUG  DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
Loop

End

'*************************************************************************
'* 3 neue Bytes vom Master empfangen
'*************************************************************************
Daten_auswertung:
   Neue_daten = 0                                 ' flag zurcksetzen

   If Schalter = 1 Then                           ' Knoten setzen
      Gosub Neu_i2c
   Elseif Chin = 0 And Schalter = 128 Then        '  * --> 128
      Select Case I
         Case 0
            Gosub Standard_speichern              ' ---> Sub
         Case 1                                   '
            Gosub Standard_laden                  ' ----> Sub
      End Select
   End If

Return
'*************************************************************************
'* I2C hat neue Daten
'*************************************************************************
Neu_i2c:
   Gosub 96bitausgabe1                            ' Matrix laden, unabhngig von Cursorposition
                                                  ' .... aber nicht mehr "Erste_zeile" NNNNNNNNNNN
   Locate 1 , I                                   ' NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN
   If Chout(i) > 9 Then
      Chout_asc = Chout(i) + 55                   ' in Ascii Buchstaben wandeln
   Else
      Chout_asc = Chout(i) + 48                   ' in Ascii Zahl wandeln
   End If
   Chout_disp = Chr(chout_asc)                    ' in Char wandeln
   Lcd Chout_disp

   Locate 2 , I

'   Gosub Erste_zeile                              '... und auf den Display UND Matrix laden
Return

'*************************************************************************
'* Tasten/Encoder - Auswertungen  (fr Handbetrieb)
'*************************************************************************
Plusplus:
   If Inout = 0 Then
      Incr I
      If I > 16 Then I = 1
      Locate Z , I                                '
   Else
      Incr Chout(i)
      If Chout(i) > 16 Then Chout(i) = 1          ' berlauf fr 16x16
      Locate Z , I

      If Chout(i) > 9 Then
         Chout_asc = Chout(i) + 55                ' in Ascii Buchstaben wandeln
      Else
         Chout_asc = Chout(i) + 48                ' in Ascii Zahl wandeln
      End If
      Chout_disp = Chr(chout_asc)                 ' in Char wandeln

      Lcd Chout_disp
      Locate Z , I
      Gosub 16bitausgabe                          ' "Vorschau" auf Ausgang 16
      Gosub 96bitausgabe0                         ' update abhngig von Cursorposition
   End If
   Plus = 0                                       ' Flag zurcksetzen
Return

Minusminus:
   If Inout = 0 Then
      Decr I
      If I < 1 Then I = 16
      Locate Z , I                                '
   Else
      Decr Chout(i)
      If Chout(i) < 1 Then Chout(i) = 16          ' berlauf fr 16x16
      Locate Z , I                                '
      If Chout(i) > 9 Then
         Chout_asc = Chout(i) + 55                ' in Ascii Buchstaben wandeln
      Else
         Chout_asc = Chout(i) + 48                ' in Ascii Zahl wandeln
      End If
      Chout_disp = Chr(chout_asc)                 ' in Char wandeln

      Lcd Chout_disp
      Locate Z , I
      Gosub 16bitausgabe                          ' "Vorschau" auf Ausgang 16
      Gosub 96bitausgabe0                         ' update abhngig von Cursorposition
   End If
   Minus = 0                                      ' Flag zurcksetzen
Return

'*************************************************************************
'* In/Out toggeln (fr Handbetrieb per Encoder)
'*************************************************************************
Einaus:
   Toggle Inout
   If Inout = 0 Then
      Z = 2
      Gosub 96bitausgabe1                         ' Matrix neu laden
   Else
      Z = 1
   End If
   Locate Z , I
Return

'*************************************************************************
'* 96 Bit Ausgabe   MODE = 1
'*************************************************************************
96bitausgabe0:                                    ' Matrix NICHT laden wenn Cursor auf Eingabekanal
   If Z = 1 Then Goto Raus1
96bitausgabe1:                                    ' Matrix immer laden unabhngig von Cursorposition
   Ce = 0
   Update = 1
   Maxmode = 1
   For N = 16 To 1 Step -1                        ' nacheinander alle Werte
      6bitdaten = Chout(n) - 1
      Shift 6bitdaten , Left , 2
      Set 6bitdaten.6                             'Verstrkung =2
      Set 6bitdaten.7                             'Ausgang anschalten
      Shiftout Din , Sclk , 6bitdaten , 0 , 6     'MSB wenn Clk = 0
   Next
   Update = 0
   Maxmode = 1
   Ce = 0
Raus1:
Return

'*************************************************************************
'* 16 Bit Ausgabe  MODE = 0
'*************************************************************************
16bitausgabe:
   Ce = 0
   Update = 1
   Maxmode = 0
   Maske = &B0000_0011_1111_0000
   16bitdaten = Chout(i) - 1
   16bitdaten = Maske Or 16bitdaten

   Shiftout Din , Sclk , 16bitdaten , 0 , 16
   Update = 0
   Maxmode = 0
   Ce = 0
Return

'********************************************************************
'* Update 1. Zeile (Input)
'********************************************************************
Erste_zeile:
   Locate 1 , 1

   For N = 1 To 16
      If Chout(n) > 9 Then
         Chout_asc = Chout(n) + 55                ' in Ascii Buchstaben wandeln
      Else
         Chout_asc = Chout(n) + 48                ' in Ascii Zahl wandeln
      End If
      Chout_disp = Chr(chout_asc)                 ' in Char wandeln
      Lcd Chout_disp                              ' ... und in upperline ausgeben
      Waitms 50                                   ' NNNNNNNNNNNNNNNNNNNN
   Next N
   Locate 2 , I
Return

'*********************************************************************
' VMuster als Standard in EEProm speichern
'*********************************************************************
Standard_speichern:
                               '
   For I = 1 To 16
      Writeeeprom Chout(i) , I                    '
      Waitms 10                                   ' wieviele ms ???
   Next I

   I = 1                                          ' Cursor wieder Position 1
Return

'**********************************************************************
' Standardkonfigurationen aus EEProm laden
'**********************************************************************
Standard_laden:                                   ' aus EEPROM

   For I = 1 To 16                                ' Var aus Data
      Readeeprom Chout(i) , I
   Next I

   I = 1                                          ' Cursor wieder Position 1
Return

'*************************************************************************
'* Interrupt Services
'*************************************************************************
Isr0:                                             ' Encoder Auswertung (Standard Encoder)
   If Pind.2 = Updown Then
      Plus = 1                                    ' Flag setzen
   Else
      Minus = 1                                   ' Flag setzen
   End If
Return
'------------------------------------
Twi_gotdata:                                      ' Daten vom Master: 3 Bytes (Befehle) oder 17 Bytes (Muster) kommen rein
   Masterbyte = Twi                               ' neue Daten merken

   If Masterbyte > 64 And Masterbyte < 128 Then   ' wegen * und # (128 und 129)
      V = Masterbyte - 64
   Else
      If Byte_count = 0 Then
         Chin = Masterbyte                        '
         Byte_count = 1
      Elseif Byte_count = 1 Then
         I = Masterbyte
         Byte_count = 2
         Chout(i) = Chin
      Elseif Byte_count = 2 Then
         Schalter = Masterbyte
         Byte_count = 0
         Neue_daten = 1                           ' 3 neue bytes vom Master empfangen
      End If
   End If
Return
'---------------------------------------------------------------------------------
Twi_master_needs_byte:                            ' Master fragt Daten ab
   Twi = Chout(v)                                 ' entweder 16 bytes (=alle Koppelpunkte mit v = 1)...
   V = V + 1                                      ' ...oder 1 byte (= Koppelpunkt v)
Return

'------------------------------ mssen im Quelltext sein (nicht benutzt) ---------------
Twi_stop_rstart_received:
   nop
Return
Twi_addressed_goread:
   nop
Return
Twi_addressed_gowrite:
   nop
Return
Twi_master_need_nomore_byte:
   nop
Return

''*************************************************************************
''* Defaultwerte zur Initialisierung der Eingabekanle
'Dta:
'$eeprom
'Data 0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10 , 10 , 10 , 10 , 10 , 10 , 10
'$data
'End
''*************************************************************************